<?php

// Helper function to instantiate a type definition file
function prosepoint_unserialize_complex_type($type_name) {
  $types_dir = PROSEPOINT_PROFILE_DIR . '/types';

  $definition_file = $types_dir . '/' . $type_name . '.type';
  $content = NULL;
  if (is_file($definition_file)) {
    $type_label = node_get_types('name', $type_name);

    $definition = file_get_contents($definition_file);
    @eval($definition);

    if (!isset($content) || !isset($content['type']) || !is_array($content) || !is_array($content['type'])) {
      drupal_set_message(t('Corrupted file @file', array('@file' => $definition_file)), 'error');
      $content = NULL;
    }
  }
  return $content;
}

// This borrows some chunks of code from content_copy_import_form_submit()
function prosepoint_update_complex_type($type_name) {
  $content = prosepoint_unserialize_complex_type($type_name);
  if (!$content)
    return FALSE;
    
  $type_label = node_get_types('name', $type_name);
  module_load_include('inc', 'content', 'includes/content.crud');

  // Get all type and field info for this database.
  $content_info = _content_type_info();

  $imported_type = $content['type'];
  $imported_type_name = $imported_type['type'];
  $imported_type_label = $imported_type['name'];

  // It is allowed to import a type with no fields,
  // so the fields array could be empty and must be cast as an array.
  $imported_fields = (array) $content['fields'];

  // Perform more pre-import error trapping.
  // If there are potential problems, exit without doing the import.
  $not_enabled = array();

  // The groups array could be empty and still valid, make sure to cast it as an array.
  // If there are groups in the import, make sure the fieldgroup module is enabled.
  if (isset($content['groups']) && module_exists('fieldgroup')) {
    $imported_groups = (array) $content['groups'];
  }
  elseif (isset($content['groups']) && is_array($content['groups'])) {
    $not_enabled[] = 'fieldgroup';
  }

  // Make sure that all the field and widget modules in the import are enabled in this database.
  foreach ($imported_fields as $import) {
    $field = content_field_instance_collapse($import);
    if (empty($field['module']) || empty($field['widget_module'])) {
      $not_enabled[] = $field['field_name'];
    }
    else {
      if (!module_exists($field['module'])) {
        $not_enabled[] = $field['module'];
      }
      if (!module_exists($field['widget_module'])) {
        $not_enabled[] = $field['widget_module'];
      }
    }
  }

  if ($not_enabled) {
    drupal_set_message(t('The following modules must be enabled for this import to work: %modules.', array(
        '%modules' => implode(', ', array_unique($not_enabled)))), 'error');
    return FALSE;
  }

  // Create the groups for this type, if they don't already exist.
  if (module_exists('fieldgroup') && isset($imported_groups)) {
    foreach ($imported_groups as $group) {
      $group_name = $group['group_name'];
      fieldgroup_save_group($type_name, $group);
    }
    // Reset the static variable in fieldgroup_groups() with new data.
    fieldgroup_groups('', FALSE, TRUE);
  }

  // Iterate through the field forms in the import and execute each.
  foreach ($imported_fields as $field) {

    // Make sure the field doesn't already exist in the type.
    // If so, do nothing, fields can't be duplicated within a content type.
    $field_name   = $field['field_name'];

    if (!empty($field['field_name']) && isset($content_info['content types'][$type_name]['fields'][$field_name])) {
      // Might need to overwrite the content type name if a new type was created.
      $field['type_name'] = $type_name;

      $field = content_field_instance_update($field);
      drupal_set_message(t('The field %field_label (%field_name) was updated for the content type %type.', array(
        '%field_label' => $field['widget']['label'], '%field_name' => $field_name, '%type' => $type_label)));
        
      // drupal_set_message(t('The imported field %field_label (%field_name) was not added to %type because that field already exists in %type.', array(
        // '%field_label' => $field_label, '%field_name' => $field_name, '%type' => $type_name)));
    }
    else {

      // Might need to overwrite the content type name if a new type was created.
      $field['type_name'] = $type_name;

      $field = content_field_instance_create($field);
      drupal_set_message(t('The field %field_label (%field_name) was added to the content type %type.', array(
        '%field_label' => $field['widget']['label'], '%field_name' => $field_name, '%type' => $type_label)));
    }
    if (module_exists('fieldgroup') && isset($imported_groups)) {
      fieldgroup_update_fields($field);
    }
  }
  return TRUE;
}

function prosepoint_config_types_simple() {
  $types = array(
    array(
      'type' => 'ifimage',
      'name' => st('Image'),
      'module' => 'node',
      'description' => st('An image (with thumbnail). This is ideal for publishing photographs or screenshots.'),
      'help' => '',
      'has_title' => 1,
      'title_label' => st('Title'),
      'has_body' => 1,
      'body_label' => st('Caption'),
      'min_word_count' => 0,
      'custom' => 0,
    ),
    array(
      'type' => 'page',
      'name' => st('Page'),
      'module' => 'node',
      'description' => st('A page, similar in form to a story, is a simple method for creating and displaying information that rarely changes, such as an "About us" section of a website. By default, a page entry does not allow visitor comments.'),
      'custom' => TRUE,
      'modified' => TRUE,
      'locked' => FALSE,
      'help' => '',
      'min_word_count' => '',
    ),
    array(
      'type' => 'story',
      'name' => st('Story'),
      'module' => 'node',
      'description' => st('A story, similar in form to a page, is ideal for creating and displaying content that informs or engages website visitors. News articles, press releases, site announcements, and informal blog-like entries may all be created with a story entry. By default, a story entry provides the ability to post comments.'),
      'custom' => TRUE,
      'modified' => TRUE,
      'locked' => FALSE,
      'help' => '',
      'min_word_count' => '',
    ),
    array(
      'type' => 'source',
      'name' => st('Source'),
      'module' => 'node',
      'description' => st('A writer or source of a story, which may or may not be the same as the person who uploads the story. This allows website staff to upload stories written by other parties, whilst still displaying proper credits.'),
      'help' => st(''),
      'has_title' => TRUE,
      'title_label' => st('Name'),
      'has_body' => TRUE,
      'body_label' => st('Description'),
      'min_word_count' => '',
      'custom' => TRUE,
    ),
    array(
      'type' => 'edition',
      'name' => st('Edition'),
      'module' => 'node',
      'description' => st('An edition is a specific issue or version of a channel. For example, an edition could be the front page (or business) news of your newspaper for January 1st, 2000, or it could be the February 2000 edition of your magazine.'),
      'help' => st(''),
      'has_title' => TRUE,
      'title_label' => st('Edition'),
      'body_label' => 'Introduction',
      'min_word_count' => '',
      'custom' => TRUE,
    ),
    array(
      'type' => 'media',
      'name' => st('Media'),
      'module' => 'node',
      'description' => st('A media file. Use this for uploading video or audio files. '),
      'help' => st(''),
      'has_title' => TRUE,
      'title_label' => st('Title'),
      'has_body' => TRUE,
      'body_label' => 'Description',
      'min_word_count' => '',
      'custom' => TRUE,
    ),
  );

  foreach ($types as $type) {
    $type = (object) _node_type_set_defaults($type);
    node_type_save($type);
  }

  // Default workflow settings by type
  variable_set('node_options_ifimage', array (
    0 => 'status',
  ));
  variable_set('node_options_media', array (
    0 => 'status',
  ));
  variable_set('node_options_page', array (
    0 => 'status', 1 => 'revision', 
  ));
  variable_set('node_options_story', array (
    0 => 'status', 1 => 'revision', 
  ));
  variable_set('node_options_source', array (
    0 => 'status',
  ));
  variable_set('node_options_edition', array (
    0 => 'status',
  ));
  /*
  variable_set('node_options_channel', array (
    0 => 'status',
  ));*/

  // Default comment settings by type
  variable_set('comment_ifimage', 0);
  variable_set('comment_media', 0);
  variable_set('comment_page', 0);
  variable_set('comment_source', 0);
  variable_set('comment_edition', 0);
  // variable_set('comment_channel', 0);
}

function prosepoint_config_types_complex() {
  $types_dir = PROSEPOINT_PROFILE_DIR . '/types';
  
  // Complex CCK type: ifimage
  // Make sure in ifimage.type that 'list_default' => '0', not 'list_default' => 0
  //   et. al. 
  $ifimage_type = file_get_contents($types_dir . '/ifimage.type');
  $data = array('values' => array('type_name' => 'ifimage', 'macro' => $ifimage_type));
  $result = drupal_execute('content_copy_import_form', $data);
  $content_extra_weights_ifimage = array('body_field' => 0, 'taxonomy' => 1, 'menu' => 1);
  variable_set('content_extra_weights_ifimage', $content_extra_weights_ifimage);
  
  // Complex CCK type: source
  $source_type = file_get_contents($types_dir . '/source.type');
  $data = array('values' => array('type_name' => 'source', 'macro' => $source_type));
  $result = drupal_execute('content_copy_import_form', $data);
  $content_extra_weights_source = array('body_field' => 0, 'menu' => 1);
  variable_set('content_extra_weights_source', $content_extra_weights_source);
  
  // Complex CCK type: story 
  $story_type = file_get_contents($types_dir . '/story.type');
  /* Tweak these in story_type ... 
     'granularity' => array('year' => 'year', 'month' => 'month', 'day' => 'day',
                            'hour' => false, 'minute' => false, 'second' => false),
     'referenceable_types' => array('writer' => 'writer', 'image' => false, 'page' => false, 'story' => false),
  */
  $data = array('values' => array('type_name' => 'story', 'macro' => $story_type));
  $result = drupal_execute('content_copy_import_form', $data);
  // refine some weights
  $content_extra_weights_story = array(
    'title' => -9,
    'taxonomy' => -4, 
    'body_field' => 0,
    'pp_ed_img_extract_preset' => 1,
    'workflow' => -4,
    'menu' => 2,
  );
  variable_set('content_extra_weights_story', $content_extra_weights_story);
  // Get around some random bug
  db_query("UPDATE {content_node_field_instance} SET widget_settings = '%s' WHERE field_name = '%s' AND type_name = '%s'", 'a:2:{s:13:"default_value";a:1:{i:0;a:2:{s:5:"value";s:1:"2";s:14:"_error_element";s:50:"default_value_widget][field_extended_status][value";}}s:17:"default_value_php";N;}', 'field_extended_status', 'story');
  
  // Complex CCK type: edition
  $edition_type = file_get_contents($types_dir . '/edition.type');
  /* Tweak these in *_type ... 
     'granularity' => array('blah' => 'blah', 'blah2' => 'blah2', 'blah3' => false, 'blah4' => false),
     'referenceable_types' => array('blah' => 'blah', 'blah2' => 'blah2', 'blah3' => false, 'blah4' => false),
  */
  $data = array('values' => array('type_name' => 'edition', 'macro' => $edition_type));
  $result = drupal_execute('content_copy_import_form', $data);
  // refine some weights
  $content_extra_weights_edition = array(
    'title' => -9,
    // 'taxonomy' => -8,
    'body_field' => -3,
    'menu' => 1,  
  );
  variable_set('content_extra_weights_edition', $content_extra_weights_edition);

  // Complex CCK type: media
  // Make sure in media.type that 'list_default' => '0', not 'list_default' => 0
  //   et. al. 
  $media_type = file_get_contents($types_dir . '/media.type');
  $data = array('values' => array('type_name' => 'media', 'macro' => $media_type));
  $result = drupal_execute('content_copy_import_form', $data);
  $content_extra_weights_media = array('body_field' => 0, 'menu' => 1);
  variable_set('content_extra_weights_media', $content_extra_weights_media);
  
  content_clear_type_cache(TRUE);
}

function prosepoint_config_nodes_simple() {
  global $node_table;
  module_load_include('inc', 'node', 'node.pages'); 

  foreach ($node_table as $k => $v) {
    $data = array('values' => $v['values']);
    // Fill in defaults
    if (!$data['values']['op']) 
      $data['values']['op'] = t('Save');
    if (!$data['values']['name']) 
      $data['values']['name'] = 'admin';      
    // Unset 'fields' just in case it upsets the node_form
    if ($data['values']['fields']) {
      unset($data['values']['fields']);
    }
    drupal_execute($v['type'] . '_node_form', $data, array('type' => $v['type']));
    
    // Save node and nid back to node_table
    if ($data['nid']) {
      $node_table[$k]['nid'] = $data['nid'];      
      $node_table[$k]['node'] = $data['node'];
      $node_table[$k]['node']['nid'] = $data['nid'];
    }
  } 
}

  // Note: vid == nid is true for the scope of this script    
function prosepoint_config_nodes_complex() { 
  global $node_table;
  global $channels_table;

  // Image / ifimage fields
  foreach ($node_table as $wrapper) {
    if ($wrapper['type'] == 'ifimage') {
      $values = $wrapper['values'];
      $fields = $values['fields'];
      $node = $wrapper['node'];

      // field_file, width and height
      $path = $fields['field_file']['file'];
      $result = file_copy($path);

      if ($result) {
        $file = _prosepoint_insert_file($path, $fields['field_file']['mime']);
        if ($file) {
          db_query("INSERT INTO {content_type_ifimage} (vid, nid, field_image_fid, field_image_list, field_image_data) VALUES (%d, %d, %d, %d, '%s')", $node['nid'], $node['nid'], $file->fid, 0, serialize(array()));
          // generate admin thumbnail
          imagefield_file_insert($file);
        }
      }
    }
  }

  // Media fields
  foreach ($node_table as $wrapper) {
    if ($wrapper['type'] == 'media') {
      $values = $wrapper['values'];
      $fields = $values['fields'];
      $node = $wrapper['node'];

      // field_file, width and height
      $path = $fields['field_file']['file'];
      $result = file_copy($path);

      if ($result) {
        $file = _prosepoint_insert_file($path, $fields['field_file']['mime']);
        if ($file) {
          db_query("INSERT INTO {content_type_media} (vid, nid, field_file_fid, field_file_list, field_file_data, field_width_value, field_height_value) VALUES (%d, %d, %d, %d, '%s', %d, %d)", $node['nid'], $node['nid'], $file->fid, 0, serialize(array()), $fields['field_width']['value'], $fields['field_height']['value']);
        }
      }
    }
  }

  // Story fields
  $workflow_states = variable_get('pp_publishing_states', FALSE);
  $public_sid = $workflow_states['public']['sid'];
  foreach ($node_table as $wrapper) {
    if ($wrapper['type'] == 'story') {
      $values = $wrapper['values'];
      $fields = $values['fields'];
      $node = $wrapper['node'];

      // Subtitle, Date
      db_query("INSERT INTO {content_type_story} (vid, nid, field_subtitle_value, field_date_value) VALUES (%d, %d, '%s', '%s')", $node['nid'], $node['nid'], $fields['field_subtitle']['value'], $fields['field_date']['value']);

      // Date
      // db_query("INSERT INTO {content_field_date} (vid, nid, field_date_value) VALUES (%d, %d, '%s')", $node['nid'], $node['nid'], $fields['field_date']['value']);
      
      // By Line
      foreach ($fields['field_source']['labels'] as $delta => $label) {
        db_query("INSERT INTO {content_field_source} (vid, nid, delta, field_source_nid) VALUES ('%d', '%d', '%d', '%d')", $node['nid'], $node['nid'], $delta, $node_table[$label]['nid']);
      }

      // Initialise workflow state to Public
      // We have to force this through because workflow.module has some outdated data due to caching
      workflow_execute_transition(node_load($node['nid']), $public_sid, NULL, TRUE);
    }   
  }

  // Edition fields
  foreach ($node_table as $wrapper) {
    if ($wrapper['type'] == 'edition') {
      $values = $wrapper['values'];
      $fields = $values['fields'];
      $node = $wrapper['node'];

      $tid = db_result(db_query("SELECT tid from {term_data} WHERE name = '%s'", $fields['field_channel']['#name']));
      if (!$tid) {
        drupal_set_message('Missing channel term ' . $terms['term_channel']['#name'], 'error');
        $tid = '';
      }
      
      // Channel, Edition id
      $channel_label = $fields['field_channel']['label'];
      db_query("INSERT INTO {content_type_edition} (vid, nid, field_channel_value, field_eid_value, field_schedule_value) VALUES (%d, %d, %d, '%s', '%s')", $node['nid'], $node['nid'], $tid, $fields['field_eid']['value'], $fields['field_schedule']['value']);

      // Date
      // db_query("INSERT INTO {content_field_date} (vid, nid, field_date_value) VALUES (%d, %d, '%s')", $node['nid'], $node['nid'], $fields['field_date']['value']);
      
      // Headlines
      foreach ($fields['field_headlines']['labels'] as $delta => $label) {
        db_query("INSERT INTO {content_field_headlines} (vid, nid, delta, field_headlines_nid, field_headlines_format) VALUES ('%d', '%d', '%d', '%d', '%s')", $node['nid'], $node['nid'], $delta, $node_table[$label]['nid'], 'teaser');
      }

      // Other Stories
      foreach ($fields['field_stories']['labels'] as $delta => $label) {
        db_query("INSERT INTO {content_field_stories} (vid, nid, delta, field_stories_nid) VALUES ('%d', '%d', '%d', '%d')", $node['nid'], $node['nid'], $delta, $node_table[$label]['nid']);
      }

      // Composite Layout
      $real_node = node_load($node['nid']);
      $real_node->composite_layout = $values['composite']['composite_layout'];
      $real_node->composite_references = $values['composite']['composite_references'];
      composite_update($real_node);      
    }
  }
  pp_ch_update_channel_editions();
  pp_update_edition_story_references_all();  
}
