<?php
/* Functions related to channel blocks */

function pp_ch_default_block_settings() {
  return array('mode' => 'list', 'view' => 'title', 'count' => 5);
}

function pp_ch_block($op = 'list', $delta = 0, $edit = array())
{
  switch ($op) {
    case 'list':
      $blocks = array();
      // Add a block for each channel 
      $terms = pp_ch_get_channels();
      foreach ($terms as $term) {
        $channel = pp_ch_load($term->tid);
        $blocks['channel_' . $term->tid] = array(
           'info' => t('Channel: @title', array('@title' => $term->name)),
          'cache' => BLOCK_NO_CACHE,
        );
      }
      return $blocks;
      break;
      
    case 'view':
      if (strpos($delta, 'channel_') === 0) {
        $tid = substr($delta, 8);
        if (is_numeric($tid) && $tid) {
          $term = taxonomy_get_term($tid);
          if (!empty($term)) {
            $settings = unserialize(db_result(db_query("SELECT block_display FROM {term_channels} WHERE tid = %d", $tid)));
            return pp_ch_channel_block_view($term, $settings);
          }
        }
      }
      break;

    case 'configure':
      // We don't use this hook. We use hook_form_alter() because we adjust the entire form, not just a subset. 
      break;      

    case 'save':
      return pp_ch_channel_block_settings_save($delta, $edit);
      break;      
  }
}

function pp_ch_channel_block_view($term, $settings) {
  $title = check_plain($term->name);
  
  if ($settings['mode'] == 'schedule') {
    // Scheduled editions channel
    $output = t('No suitable edition found.');
    $node = _pp_ch_get_latest_edition($term->tid);
    // set $page to true so the node title doesn't show
    if ($node->status) {
      if ($settings['view'] == 'title') {
        $output = l($node->title, 'node/' . $node->nid);
      }
      else /* ($settings['view'] == 'teaser') */ {
        $output = node_view($node, TRUE, TRUE, FALSE);
        // Manually add a "read more" link if we aren't already on that page. 
        if (arg(0) != 'channel' || arg(1) != $term->tid) {
          $output .= theme('channel_more_link', t('more'), 'channel/' . $term->tid);
        }
      }
    }
  }
  else if ($settings['mode'] == 'list') {
    // Story listing channel
    $output = t('No suitable stories found.');
    if ($view = views_get_view('story_view')) {
      $args = isset($term->tid) ? array($term->tid) : array();
      $display = $settings['view'] == 'teaser' ? 'block_2' : 'block_1';
      $view->set_display($display);
      $view->display_handler->set_option('items_per_page', $settings['count']);
      $result = $view->execute_display($display, $args);
      if ($result['content']) {
        $output = $result['content'];
        $title = $result['subject'];
      }
      $view->destroy();
    }
    else {
      drupal_set_message(t('ProsePoint error: Missing view %view', array('%view' => 'story_view')), 'error');
    }
  }
  else if ($settings['mode'] == 'edition') {
    $output = t('No suitable edition found.');
    $node = node_load($settings['edition_nid']);
    if ($node->status) {
      if ($settings['view'] == 'title') {
        $output = l($node->title, 'node/' . $node->nid);
      }
      else /* ($settings['view'] == 'teaser') */ {
        $output = node_view($node, TRUE, TRUE, FALSE);
        // Manually add a "read more" link if we aren't already on that page. 
        if (arg(0) != 'channel' || arg(1) != $term->tid) {
          $output .= theme('channel_more_link', t('more'), 'channel/' . $term->tid);
        }
      }
    }
  }
  else if ($settings['mode'] == 'stories') {
    // Create a dummy node because formatters need it
    $fake_node = new StdClass();
    $fake_node->nid = 0;

    // Sort items according to ['_weight']
    usort($settings['field_headlines'], '_content_sort_items_helper');
    usort($settings['field_stories'], '_content_sort_items_helper');
    
    // Render the headlines. Use pp_anr_formatters if present, otherwise use node_view()
    if (function_exists('theme_pp_anr_formatter_defer')) {
      foreach ($settings['field_headlines'] as $item) {
        $element = array('#item' => $item, '#node' => $fake_node);
        // ToDo: Should probably theme-ify the div wrapper
        $output .= '<div>' . theme('pp_anr_formatter_defer', $element) . '</div>';
      }
    }
    else {
      // Fallback to node_view()
      foreach ($settings['field_headlines'] as $item) {
        $node = node_load($item['nid']);
        if ($node->status && node_access('view', $node)) {
          $output .= node_view($node, TRUE);
        }
      }
    }
    
    // Render the non-headlines. Use nodereference_formatters if present, otherwise use node_view()
    $list = array();
    if (function_exists('theme_nodereference_formatter_default')) {
      foreach ($settings['field_stories'] as $item) {
        $element = array('#item' => $item, '#node' => $fake_node);
        $list[] = theme('nodereference_formatter_default', $element);
      }
    }
    else {
      foreach ($settings['field_stories'] as $item) {
        $node = node_load($item['nid']);
        if ($node->status) {
          $list[] = l($node->title, 'node/' . $node->nid);
        }
      }
    }
    $output .= theme('item_list', $list);
  }
  else {
    drupal_set_message(t('ProsePoint error: Unknown block display mode %mode', array('%mode' => $settings['mode'])), 'error');
  }
  return array('subject' => $title, 'content' => $output);
}

function pp_ch_channel_block_settings_form_alter(&$form, &$form_state) {
  $delta = $form['delta']['#value'];
  $tid = substr($delta, 8);
  $result = db_result(db_query("SELECT block_display FROM {term_channels} WHERE tid = %d", $tid));
  $settings = $result ? unserialize($result) : pp_ch_default_block_settings();

  $form['block_display'] = pp_ch_channel_block_settings_subform($settings, $tid);
  
  $form['block_display']['#type'] = 'fieldset';
  $form['block_display']['#title'] = t('Block display settings');
  $form['block_display']['#tree'] = FALSE;
  $form['block_display']['#collapsible'] = TRUE;
  
  // Adjust ordering of fieldsets.
  $form['block_settings']['#weight'] = -10;
  $form['block_display']['#weight'] = -9;

  // Add validate handler
  $form['#validate'][] = 'pp_ch_channel_block_settings_validate';

  return $form;
}

// Broken up into a helper function so others can use it. 
function pp_ch_channel_block_settings_subform($settings, $tid) {
  drupal_add_js(drupal_get_path('module', 'pp_ch') . '/pp_ch.block_settings.js', 'module', 'header', FALSE, TRUE, FALSE);

  $form = array();
  
  // To consider: An arbitrary invariant list of stories? 
  $mode_options = array(
    'list' => t('A reverse chronological list of stories in this channel, most recent first'),
    'schedule' => t('The current scheduled edition of this channel'),
    'edition' => t('A specific edition of this channel'),
    'stories' => t('A selection of headlines and non-headlines stories'),
  );  
  $form['mode'] = array(
    '#type' => 'select',
    '#title' => 'What should this display?',
    '#options' => $mode_options,
    '#default_value' => $settings['mode'],
  );

  $view_options = array(
    'title' => t('Title'),
    'teaser' => t('Teaser'),
  );
  $form['view'] = array(
    '#type' => 'select',
    '#title' => 'Display them as?',
    '#options' => $view_options,
    '#default_value' => $settings['view'],
  );

  $count_options = array_combine(range(1, 20), range(1, 20));
  $form['count'] = array(
    '#type' => 'select',
    '#title' => t('Number of stories to display'),
    '#description' => t('This only applies if reverse chronological list of stories is selected above.'),
    '#options' => $count_options,
    '#default_value' => $settings['count'],
  );

  // $editions_options
  if ($view = views_get_view('channel_editions_list')) {
    $view->set_arguments(array($tid));
    $result = $view->execute();
    $results = $view->result;
    $view->destroy();

    $editions_options = array();
    foreach ($results as $item) {
      $editions_options[$item->nid] = $item->node_data_field_channel_field_eid_value;
    }

    $form['edition_nid'] = array(
      '#type' => 'select',
      '#title' => t('Edition'),
      '#description' => t('The edition to display'),
      '#options' => $editions_options,
      '#default_value' => $settings['edition_nid'],
    );
  }  
  else {
    drupal_set_message(t('ProsePoint error: Missing view %view', array('%view' => 'channel_editions_list')), 'error');
  }

  // Okay, this is naughty. We are going to pretend we're an edition node
  //   with field_headlines and field_stories and hope CCK doesn't notice or complain :)
  module_load_include('inc', 'content', 'includes/content.node_form');

  $field1 = content_fields('field_headlines', 'edition');
  $form['field_headlines'] = content_multiple_value_form($form, $form_state, $field1, $settings['field_headlines']);
  $form['field_headlines']['#field_name'] = $field1['field_name'];
  $form['field_headlines']['#tree'] = TRUE;

  $field2 = content_fields('field_stories', 'edition');
  $form['field_stories'] = content_multiple_value_form($form, $form_state, $field2, $settings['field_stories']);
  $form['field_stories']['#field_name'] = $field2['field_name'];
  $form['field_stories']['#tree'] = TRUE;

  return $form;
}

function pp_ch_channel_block_settings_validate($form, &$form_state) {
  // dummy node
  $node = array();

  // Get nodereference to do validation for us. 
  if ($form_state['values']['mode'] == 'stories') {
    nodereference_field('validate', $node, content_fields('field_headlines', 'edition'), $form_state['values']['field_headlines'], NULL, NULL);
    nodereference_field('validate', $node, content_fields('field_stories', 'edition'), $form_state['values']['field_stories'], NULL, NULL);
  }
}


function pp_ch_channel_block_settings_save($delta, $edit) {
  $tid = substr($delta, 8);

  // Compose the settings we want to save
  $data = array(
    'mode' => $edit['mode'],
    'view' => $edit['view'],
    'count' => $edit['count'],
    'edition_nid' => $edit['edition_nid'],
  );
  $data['field_headlines'] = $edit['field_headlines'];
  $data['field_stories'] = $edit['field_stories'];

  // Delete the stuff we don't want to save
  unset($data['field_headlines']['field_headlines_add_more']);
  foreach ($data['field_headlines'] as $k => $v) {
    unset($data['field_headlines'][$k]['_error_element']);
  }

  unset($data['field_stories']['field_stories_add_more']);
  foreach ($data['field_stories'] as $k => $v) {
    unset($data['field_stories'][$k]['_error_element']);
  }

  // ToDo: warn if mode=schedule for story listing channel
  db_query("UPDATE {term_channels} SET block_display = '%s' WHERE tid = %d", serialize($data), $tid);
}


