<?php
/*
 * Copyright 2008 ThinkLeft (thinkleft.com.au)
 * Copyright 2008 ProsePoint (www.prosepoint.org)
 */

// vim: filetype=php
define("PROSEPOINT_PROFILE_DIR", 'profiles/prosepoint');
include_once(PROSEPOINT_PROFILE_DIR . '/install_profile_api/core/block.inc');
include_once(PROSEPOINT_PROFILE_DIR . '/install_profile_api/core/user.inc');

function prosepoint_verify_requirements() {
  /*
  // These checks are for the tinymce compressor
  $version = phpversion();
  $zlib_compression = ini_get('zlib.output_compression');
  if (version_compare(phpversion(), '5') >= 0 && !$zlib_compression) {
    return true;
  }
  else {
    return false;
  }
  */
  return true;
}

/* Set up filesystem */
function prosepoint_config_files() {
  variable_set('file_downloads', '1');  
}

// Adds a file to the {files} table and returns the fid. uid defaults to current user
function _prosepoint_insert_file($filename, $mime, $uid = 'user') {
  if ($uid == 'user') {
    global $user;
    $uid = $user->uid;
  }

  $file = new stdClass();
  $file->filename = basename($filename);
  $file->filepath = $filename;
  $file->filemime = $mime;
  $file->filesize = filesize($filename);

  $file->uid = $uid;
  $file->status = FILE_STATUS_PERMANENT;
  $file->timestamp = time();
  drupal_write_record('files', $file);
  $file->fid = db_result(db_query("SELECT fid FROM {files} WHERE filepath = '%s'", $file->filepath));
  return $file;
}

function prosepoint_config_date() {
  // Set general defaults
  variable_set('date_default_timezone', 28800);
  variable_set('date_default_timezone_name', 'Australia/Perth');
  variable_set('date_first_day', '0');
  variable_set('date_format_long', 'l, F j, Y - H:i');
  variable_set('date_format_medium', 'D, d/m/Y - H:i');
  variable_set('date_format_short', 'd/m/Y - H:i');

  // Add custom format 'l, F jS, Y'
  $format = array();
  $format['format'] = 'l, F jS, Y';
  $format['type'] = 'custom';
  $format['locked'] = 0;
  $format['is_new'] = 1;
  date_format_save($format);

  // Add new date format type
  $format_type = array();
  $format_type['title'] = t('Story Date');
  $format_type['type'] = 'story_date';
  $format_type['locked'] = 1;
  $format_type['is_new'] = 1;
  date_format_type_save($format_type);
  variable_set('date_format_story_date', $format['format']);
}  

/* Set up input filters */
function prosepoint_config_filters() {
  variable_set('allowed_html_1', '<a> <em> <strong> <cite> <code> <ul> <ol> <li> <dl> <dt> <dd> <p> <span> <img> <div> <pp_img> <pp_media> <h2> <h3> <h4> <h5> <h6> <br> <blockquote> <table> <tbody> <tr> <th> <td>');
  // Note: The fid of 10 is a hardcoded value
  db_query("INSERT INTO {filters} (fid, format, module, delta, weight) VALUES (8, 1, 'pp_ed', 0, 10), (9, 2, 'pp_ed', 0, 10)");
  // Remove line break converter
  db_query("DELETE FROM {filters} WHERE module = 'filter' AND delta = 1");

  // Allow editor and administrator roles to use Full HTML
  $allowed_roles = array();
  $result = db_query("SELECT rid FROM {role} WHERE name = '%s' OR name = '%s'", 'editor', 'administrator');
  while ($object = db_result($result)) {
    $allowed_roles[] = $object;
  }
  $role_string = ',' . implode(',', $allowed_roles) . ',';
  db_query("UPDATE {filter_formats} SET roles = '%s' WHERE format = %d", $role_string, 2);
}

function prosepoint_config_menus() {
  db_query("UPDATE {menu_custom} SET title = '%s', description = '%s' WHERE menu_name = '%s'", t('Main Menu'), t('Main Menu'), 'primary-links');
  db_query("UPDATE {menu_custom} SET title = '%s', description = '%s' WHERE menu_name = '%s'", t('Info Menu'), t('Info Menu'), 'secondary-links');
  variable_set('menu_primary_links_source', 'primary-links');
  variable_set('menu_secondary_links_source', 'primary-links');
  }

function prosepoint_config_contact() {
  variable_set('contact_default_status', 0);
  
  // Set up a default contact category pointing to the superuser's email
  $admin = user_load(1);
  db_query("INSERT INTO {contact} (category, recipients, reply, weight, selected) VALUES ('%s', '%s', '%s', %d, %d)", t('General'), $admin->mail, '', 0, 1);
}

function prosepoint_config_image_settings() {
  $presets = array(
    array(
      'presetname' => 'square',
      'actions' => array(
        array(
          'action' => 'imagecache_scale_and_crop',
          'data' => array('width' => 150, 'height' => 150),
          'weight' => 0,
          'module' => 'imagecache',       
        ),
      ),
    ),
    array(
      'presetname' => 'thumbnail',
      'actions' => array(
        array(
          'action' => 'imagecache_scale',
          'data' => array('width' => 75, 'height' => '', 'upscale' => 0),
          'weight' => 0,
          'module' => 'imagecache',       
        ),
      ),
    ),
    array(
      'presetname' => 'small',
      'actions' => array(
        array(
          'action' => 'imagecache_scale',
          'data' => array('width' => 150, 'height' => '', 'upscale' => 0),
          'weight' => 0,
          'module' => 'imagecache',       
        ),
      ),
    ),
    array(
      'presetname' => 'medium',
      'actions' => array(
        array(
          'action' => 'imagecache_scale',
          'data' => array('width' => 300, 'height' => '', 'upscale' => 0),
          'weight' => 0,
          'module' => 'imagecache',       
        ),
      ),
    ),
    array(
      'presetname' => 'large',
      'actions' => array(
        array(
          'action' => 'imagecache_scale',
          'data' => array('width' => 500, 'height' => '', 'upscale' => 0),
          'weight' => 0,
          'module' => 'imagecache',       
        ),
      ),
    ),
  );

  foreach ($presets as $preset) {
    drupal_write_record('imagecache_preset', $preset);
    foreach ($preset['actions'] as $action) {
      $action['presetid'] = $preset['presetid'];
      drupal_write_record('imagecache_action', $action);
    }
  }

  // Flush 
  imagecache_presets(true);
}

/* Set up theme related config */
function prosepoint_config_theme() {
  $theme_default = 'tma2';
  system_theme_data();
  db_query("UPDATE {system} SET status = 1 WHERE type = 'theme' and name = '%s'", 'tma2');

  $theme_settings = array(
    'toggle_logo' => 0,
    'toggle_name' => 1,
    'toggle_slogan' => 1,
    'toggle_mission' => 1,
    'toggle_node_user_picture' => 0,
    'toggle_comment_user_picture' => 0,
    'toggle_search' => 1,
    'toggle_favicon' => 1,
    'toggle_primary_links' => 1,
    'toggle_secondary_links' => 1,
    'toggle_node_info_ifimage' => 0,
    'toggle_node_info_media' => 0,
    'toggle_node_info_page' => 0,
    'toggle_node_info_story' => 0,
    'toggle_node_info_source' => 0,
    'toggle_node_info_edition' => 0,
    'default_logo' => 1,
    'logo_path' => '',
    'logo_upload' => '',
    'default_favicon' => 1,
    'favicon_path' => '',
    'favicon_upload' => '',
  );
  variable_set('theme_settings', $theme_settings);
  
  $theme_tma2_settings = array(  
    'toggle_name' => 1,
    'toggle_slogan' => 1,
    'toggle_mission' => 1,
    'toggle_node_user_picture' => 0,
    'toggle_comment_user_picture' => 0,
    'toggle_search' => 1,
    'toggle_favicon' => 1,
    'toggle_primary_links' => 1,
    'toggle_secondary_links' => 1,
    'default_favicon' => 1,
    'favicon_path' => '',
    'favicon_upload' => '',
    'zen_block_editing' => 1,
    'zen_breadcrumb' => 'yes',
    'zen_breadcrumb_separator' => ' // ',
    'zen_breadcrumb_home' => 1,
    'zen_breadcrumb_trailing' => 0,
    'zen_breadcrumb_title' => 0,
    'zen_breadcrumb_leading' => 1,
    'zen_wireframes' => 0,
    'zen_local_task_tabs' => 0,
  );
  variable_set('theme_tma2_settings', $theme_tma2_settings);
  
  variable_set('theme_default', $theme_default);
  system_initialize_theme_blocks($theme_default);
}

function prosepoint_config_blocks() {
  install_add_block('user', 0, 'bluemarine', 1, 0, 'right');
  install_add_block('user', 1, 'bluemarine', 1, 0, 'right');
  install_add_block('user', 0, 'tma2', 1, 0, 'right');
  install_add_block('user', 1, 'tma2', 1, 0, 'right');

  // Secondary links 
  install_add_block('menu', 'secondary-links', 'tma2', 1, 0, 'header_right', 0, '', 0, 0, '<none>');

  // Local Tasks Block and Channels dashboard
  install_add_block('pp_ch', 0, 'tma2', 1, 0, 'local_tasks', 0, '', 0, 0, '<none>');

  // Other stuff
  install_add_block('pp_ed', 0, 'tma2', 0, 10, '');
}

function prosepoint_config_misc() {
  // pathauto
  variable_set('pathauto_user_pattern', '');
  variable_set('pathauto_tracker_pattern', '');
  variable_set('pathauto_node_pattern', '');
  variable_set('pathauto_node_edition_pattern', 'edition/[title-raw]/[nid]');
  variable_set('pathauto_node_ifimage_pattern', 'image/[title-raw]');
  variable_set('pathauto_node_media_pattern', 'media/[title-raw]');
  variable_set('pathauto_node_page_pattern', 'page/[title-raw]');
  variable_set('pathauto_node_story_pattern', 'story/[title-raw]');
  variable_set('pathauto_node_source_pattern', 'source/[title-raw]');

  variable_set('pathauto_taxonomy_pattern', '');
  
  // submitagain
  variable_set('submitagain_story', 1);
  
  // node_clone
  variable_set('clone_method', 'prepopulate');
  variable_set('clone_omitted', array('ifimage' => 'ifimage', 'page' => 'page', 'story' => 'story', 'media' => 'media', 'source' => 'source', 'edition' => 0));
  variable_set('clone_reset_edition', '1');
  variable_set('clone_reset_ifimage', '1');
  variable_set('clone_reset_page', '1');
  variable_set('clone_reset_story', '1');
  variable_set('clone_reset_source', '1');
    
  // service_links
  variable_set('service_links_style', '2');
  variable_set('service_links_in_node', '0');
  variable_set('service_links_in_links', '2');
  variable_set('service_links_node_types', array(
    'story' => 'story', 'edition' => 0, 'ifimage' => 0, 'page' => 0, 'source' => 0,
  )); 
  variable_set('service_links_show_yahoo', 1);
  variable_set('service_links_show_technorati', 1);
  variable_set('service_links_show_stumbleupon', 1);
  variable_set('service_links_show_reddit', 1);
  variable_set('service_links_show_propeller', 0);
  variable_set('service_links_show_newsvine', 1);
  variable_set('service_links_show_magnoliacom', 1);
  variable_set('service_links_show_icerocket', 0);
  variable_set('service_links_show_google', 1);
  variable_set('service_links_show_furl', 1);
  variable_set('service_links_show_facebook', 1);
  variable_set('service_links_show_digg', 1);
  variable_set('service_links_show_delicious', 1);

  // pp_ed image extraction
  variable_set('pp_img_extract_settings', array(
    'presets' => array(
      'page' => 'small',
      'story' => 'small',
    ),
    'weight' => 0,
  ));
  
  // pp_ig image gallery
  variable_set('pp_ig_gallery_thumbnail_preset', 'square');

  // diff
  foreach (array('edition', 'forum', 'ifimage', 'media', 'page', 'story', 'source') as $type) {
    variable_set('show_preview_changes_' . $type, 0);
  }

  // composite
  variable_set('composite_enabled_edition', 1);
  
  // other stuff
  variable_set('pp_ed_caption_space', 3);
  variable_set('pp_show_story_list_for_source', 1);

  cache_clear_all('variables', 'cache');
}

function prosepoint_form_alter(&$form, $form_state, $form_id) {
  if ($form_id == 'install_configure') {
    // Set default for site name field.
    $form['site_information']['site_name']['#default_value'] = t('Your ProsePoint Site');

    $form['filesystem'] = array(
      '#title' => t('File System'),
      '#type' => 'fieldset',
    );
    $form['filesystem']['file_directory_path'] = array(
      '#type' => 'textfield',
      '#title' => t('File system path'),
      '#default_value' => 'sites/default/files',
      '#maxlength' => 255,
      '#description' => t('A file system path where the files will be stored. This directory must exist and be writable by Drupal. If the download method is set to public, this directory must be relative to the Drupal installation directory and be accessible over the web. If the download method is set to private, this directory should not be accessible over the web. Changing this location will modify all download paths and may cause unexpected problems on an existing site.'),
      '#required' => TRUE,
      '#after_build' => array('system_check_directory'),
    );
    $form['filesystem']['file_directory_temp'] = array(
      '#type' => 'textfield',
      '#title' => t('Temporary directory'),
      '#default_value' => '/tmp',
      '#maxlength' => 255,
      '#description' => t('A file system path where uploaded files will be stored during previews.'),
      '#required' => TRUE,
      '#after_build' => array('system_check_directory'),
    );
    $form['submit']['#submit'][] = '_prosepoint_install_configure_submit';
    $form['#validate'][] = '_prosepoint_install_configure_validate';

    // Development hooks, for convenience only
    if (file_exists('../maintenance/install_configure.prefill')) {
      eval(file_get_contents('../maintenance/install_configure.prefill'));
    }
  }
}

function _prosepoint_install_configure_validate($form, &$form_state) {
  if (!$form_state['values']['clean_url']) {
    form_set_error('clean_url', t('ProsePoint requires a web server that supports clean urls. Please see !link for more information.', array('!link' => '<a href="http://www.prosepoint.org/docs/web-server-requirements.html">the ProsePoint requirements pages</a>')));
  }
}

function _prosepoint_install_configure_submit($form, &$form_state) {
  if ($form_state['values']['file_directory_path'])
    variable_set('file_directory_path', $form_state['values']['file_directory_path']);

  if ($form_state['values']['file_directory_temp'])
    variable_set('file_directory_temp', $form_state['values']['file_directory_temp']);

  // Hmmm... have to call the proper submit handler ourselves? 
  install_configure_form_submit($form, $form_state);
}

function prosepoint_profile_modules() {
  return array(
    // from default.profile
    'color', 
    'comment', 
    'help', 
    'menu', 
    'taxonomy', 
    'dblog',
    
    // other drupal core
    'contact',
    'path',
    'tracker',
    'search',
    
    // CCK
    'content',
    'content_copy',
    'fieldgroup',
    'nodereference',
    'number',
    'optionwidgets',
    'text',
    'userreference',
    'filefield',
    
    'date_api',
    'date_php4',
    'date_timezone',
    'date',
    'date_popup',

    'content_taxonomy',
    'content_taxonomy_options',

    'imageapi',
    'imageapi_gd',
    'imageapi_imagemagick',
    'imagecache',
    'imagefield',
    
    // other contributed modules
    'views',
    'admin_menu',
    // Watch out, these two modules potentially conflict with admin_menu.
    'token',
    'pathauto',    
    'menu_breadcrumb',
    'submitagain',
    'clone',
    'service_links',
    'diff',
    'composite',
    'composite_cck',
    'poormanscron',
    'workflow',
    'workflow_access',    
    
    // ProsePoint
    'pp',
    'pp_ch',
    'pp_ed',
    'pp_media',
    'pp_image',
    'pp_ig',
    'pp_cs',
    'pp_img_extract',
    'pp_anr',
    'workflow_state',
  );
}

function prosepoint_profile_details() {
  return array (
    'name' => 'ProsePoint',
    'description' => 'ProsePoint content management system',
  );
}

function prosepoint_profile_task_list() {
  return array(
    'pp1' => t('Verify ProsePoint requirements'),
    'pp2' => t('Choose ProsePoint variant'),
  );
}

function prosepoint_variants_form(&$form_state, $url) {
  $form['variant'] = array(
    '#title' => t('Which variant of ProsePoint would you like to install?'),
    '#type' => 'radios',
    '#options' => array(
      'demo' => t('A demo site. Use this for evaluating ProsePoint.'),
      'minimal' => t('A site with minimal content. Use this for building your site upon.'),
    ),
    '#default_value' => 'demo',
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => st('Continue'),
    '#weight' => 15,
  );
  $form['#action'] = $url;
  $form['#redirect'] = FALSE;
  return $form;
}

function prosepoint_variants_form_submit($form, &$form_state) {
  variable_set('prosepoint_variant', $form_state['values']['variant']);
}

function prosepoint_profile_tasks(&$task, $url) {  
  switch ($task) {
    case 'profile':
    case 'pp1':
      if (!prosepoint_verify_requirements()) {
        $task = 'pp1';
        drupal_set_title(t('ProsePoint requirements not met.'));
        $output = t('ProsePoint requires PHP 5 and zlib compression in php.ini disabled. <br />Please correct this and reinstall.');
        return $output;
      }
      $task = 'pp2';
      drupal_set_title(t('Choose ProsePoint variant'));
      $output = drupal_get_form('prosepoint_variants_form', $url);
      return $output;
      break;
      
    case 'pp2':
      // Slightly buggy? Have to call the form again in order for the submit handler to trigger
      drupal_get_form('prosepoint_variants_form', $url);
      $variant = variable_get('prosepoint_variant', 'minimal');
      prosepoint_profile_install($variant);
      variable_del('prosepoint_variant');
      $task = 'profile-finished';
      break;    
      
    case 'pp_fail':
      drupal_set_title(t('ProsePoint requirements not met.'));
      $output = t('ProsePoint requires PHP 5 and zlib compression in php.ini off. <br />Please correct this and reinstall.');
      return $output;
      break;
  }
}

function prosepoint_profile_install($variant) {  
  global $node_table;
  variable_set('prosepoint_state', 'install');
  
  $variant_filename = PROSEPOINT_PROFILE_DIR . '/variants/prosepoint-variant-'. $variant .'.php';
  if (file_exists($variant_filename)) {
    include_once($variant_filename);
  }
  else {
    drupal_set_message(t('Variant definition file @filename not found', array('@filename' => $variant_filename )));
    return;
  }
  include_once(PROSEPOINT_PROFILE_DIR . '/includes/prosepoint.taxonomy.inc');
  include_once(PROSEPOINT_PROFILE_DIR . '/includes/prosepoint.content.inc');
  include_once(PROSEPOINT_PROFILE_DIR . '/includes/prosepoint.permissions.inc');
  include_once(PROSEPOINT_PROFILE_DIR . '/includes/prosepoint.workflow.inc');
    
  prosepoint_config_files();
  prosepoint_config_date();
  prosepoint_config_permissions();
  // filters must come after roles / permissions 
  prosepoint_config_filters();
  prosepoint_config_menus();

  // For some reason, drupal_execute-ing node forms doesn't work with complex CCK values
  // We encounter all sorts of unpredictable widget / validation errors
  prosepoint_config_types_simple();
  prosepoint_config_node_init();
  prosepoint_config_image_settings();
  prosepoint_config_misc();
  prosepoint_config_workflow();

  // Temporarily disable pathauto for taxonomy paths
  // variable_set('pathauto_taxonomy_pattern', '');

  if (!empty($node_table)) {
    prosepoint_config_nodes_simple();
  }
  else {
    drupal_set_message('Node table not found.');
  }
  prosepoint_config_channels();
  prosepoint_config_taxonomy();
  prosepoint_config_types_complex();

  if (!empty($node_table)) {
    prosepoint_config_nodes_complex();
  }

  node_access_rebuild();
  
  prosepoint_config_contact();

  // Re-enable pathauto for taxonomy paths
  // variable_set('pathauto_taxonomy_pattern', 'category/[vocab-raw]/[catpath-raw]');

  // Site information
  variable_set('site_slogan', t('Another ProsePoint site'));
  variable_set('site_mission', t('Welcome to ProsePoint'));
  variable_set('site_footer', t('Powered by !ProsePoint', array('!ProsePoint' => l('ProsePoint', 'http://www.prosepoint.org', array('attributes' => array('title' => t('Online newspaper and magazine software')))))));

  // Performance settings
  variable_set('preprocess_css', 1);
  // variable_set('cache', 1);

  // User settings
  variable_set('user_register', 0);

  // Update settings
  variable_set('update_notification_threshold', 'security');
        
  prosepoint_config_theme();
  prosepoint_config_blocks();
  
  // Make things developer friendly if devel.module is present
  if (file_exists(PROSEPOINT_PROFILE_DIR . '/modules/devel')) {
    drupal_set_message(t('Devel module detected. Performing extra configuration.'));
    // variable_set('admin_theme', 'bluemarine');
    module_enable(array('devel', 'imagecache_ui', 'views_ui'));
    // install_add_block('devel', 2, 'tma2', 1, 4, 'left');
    install_add_block('menu', 'devel', 'tma2', 1, 5, 'right');
    variable_set('preprocess_css', 0);
  }

  if (function_exists('prosepoint_config_extras'))
    prosepoint_config_extras();

  // DO NOT call drupal_flush_all_caches() after enabling themes. It will eventually
  //   call system_theme_data() and _system_theme_data() which reverts the list of enabled themes. 
  // drupal_flush_all_caches();

  variable_del('prosepoint_state');
}

// This is a workaround for some code on cck's *.install files
drupal_get_filename('module', 'filefield', PROSEPOINT_PROFILE_DIR . '/modules/filefield/filefield.module');
drupal_get_filename('module', 'content', PROSEPOINT_PROFILE_DIR . '/modules/cck/content.module');

